/************************************************************
 *  Martin Toepfer     ***     PROJECT : Lingvisto          *
 *  Wissmannstr. 5     ***                                  *
 *  D - 22041 Hamburg  ***     File    : xawfunction.c      *
 *  Germany            ***     Date    : 04.04.1997         *
 ************************************************************/
#include <stdlib.h>
#include <string.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include "datastructure.h"
#include "edit.h"
#include "error.h"
#include "xawfunction.h"

/************************************************************
 *      global variables (in datastructure.c)               *
 ************************************************************/
extern struct lvo_file head;

/************************************************************
 *         global variables (in xaw.c)                      *
 ************************************************************/
extern String trans1, trans2, trans5;
extern Widget edit, file, foreign, fwd, native, rev, statline, test, top; 
extern XtAppContext app_con;

/**********************************************************************
 *                    some basic X11 functions                        *
 **********************************************************************/
/************************************************************
 * SetAllSensitive : set sensitiveness of all buttons       *
 *                                                          *
 * b : True of False (sensitive or insensitive)             *
 ************************************************************/
void SetAllSensitive(Boolean b) {
  XtVaSetValues(file, XtNsensitive, b, NULL);
  XtVaSetValues(edit, XtNsensitive, b, NULL);
  XtVaSetValues(test, XtNsensitive, b, NULL);
  XtVaSetValues(fwd,  XtNsensitive, b, NULL);
  XtVaSetValues(rev,  XtNsensitive, b, NULL);
}

/************************************************************
 * ComputePsh : compute location of a popup shell           *
 ************************************************************/
Widget ComputePsh(void) {
  Widget psh;
  Position x, y;
  Dimension height, width;
  XtVaGetValues(top,
		XtNwidth, &width,
		XtNheight, &height,
		NULL);
  XtTranslateCoords(top,
		    (Position) width/4,
		    (Position) height/4,
		    &x, &y);
  psh = XtVaCreatePopupShell("", transientShellWidgetClass, top, NULL);
  XtVaSetValues(psh, 
		XtNx, x, 
		XtNy, y,
		NULL);
  SetAllSensitive((Boolean) False);
  return(psh);
}

/**********************************************************************
 *        some X11 functions to display strings in widgets            *
 **********************************************************************/
/************************************************************
 * DisplayNodeNative : display the native language word of  *
 *                     NODE n in widget 'native'            *
 *                                                          *
 * n : NODE to display                                      *
 ************************************************************/
void DisplayNodeNative(NODE n) {
  if (n != NULL)
    XtVaSetValues(native, XtNstring, n->pair.native, NULL);
  else
    XtVaSetValues(native, XtNstring, "", NULL);
}

/************************************************************
 * DisplayNodeForeign : display the foreign language word   *
 *                      of NODE n in widget 'foreign'       *
 *                                                          *
 * n : NODE to display                                      *
 ************************************************************/
void DisplayNodeForeign(NODE n) {
  if (n != NULL)
    XtVaSetValues(foreign, XtNstring, n->pair.foreign, NULL);
  else
    XtVaSetValues(foreign, XtNstring, "", NULL);
}

/************************************************************
 * DisplayState : display string in statline                *
 *                                                          *
 * state : string to display                                *
 ************************************************************/
void DisplayState(char *state) {
  XtVaSetValues(statline, XtNlabel, state, NULL);
}

/**********************************************************************
 *          some X11 functions to read strings from widgets           *
 **********************************************************************/
/************************************************************
 * DoNextEvent : process next event in XEventQueue          *
 ************************************************************/
void DoNextEvent(void) {
  extern XtAppContext app_con;
  XEvent event;
  XtAppNextEvent(app_con, &event);
  XtDispatchEvent(&event);
}

/************************************************************
 * GetString : read string from textWidget                  *
 *                                                          *
 * w : widget to read the string from                       *
 ************************************************************/
char *GetString(Widget w) {
  char *help;
  String str = "";
  unsigned char len;
  XtVaSetValues(w,
		XtNstring, "",
		XtNeditType, XawtextEdit,
		XtNdisplayCaret, True,
		NULL);
  
  /*
   * set keyboard focus to widget 'w'
   */
  
  XtSetKeyboardFocus(top, w);
  
  /*
   * get chars until 'return' is pressed
   */
  
  while (strchr(str, '\n') == NULL) {
    DoNextEvent();
    XtVaGetValues(w, XtNstring, &str, NULL);
  }
  
  /*
   * set length of string to min{MAXLINE, strlen(str)}
   */
  
  len = (unsigned char) strlen(str);
  if (len >= MAXLINE) {
    len = MAXLINE;
    lvo_error(LONG_INPUT);
  }

  /*
   * terminate String 'str'
   */
  
  str[len - 1] = '\0';
  
  /*
   * copy string
   */
  
  help = (char *) malloc(strlen(str)+1);
  if (help != NULL)
    strcpy(help, str);
  else
    lvo_error(NO_MEMORY);

  /*
   * display string
   */
  
  XtVaSetValues(w,
		XtNeditType, XawtextRead,
		XtNstring, help,
		XtNdisplayCaret, False,
		NULL);

  /*
   * set keyboard focus to toplevel widget 'top'
   */
  
  XtSetKeyboardFocus(top, top);
  
  /*
   * return result string
   */

  return(help);
}

/************************************************************
 * GetNative : read native string from 'native' widget      *
 ************************************************************/
char *GetNative(void) {
  char *help;
  XtOverrideTranslations(native,
			 XtParseTranslationTable(trans2));
  help = GetString(native);
  XtOverrideTranslations(native,
			 XtParseTranslationTable(trans1));
  return(help);
}

/************************************************************
 * GetForeign : read foreign string from 'foreign' widget   *
 ************************************************************/
char *GetForeign(void) {
  char *help;
  XtOverrideTranslations(foreign,
			 XtParseTranslationTable(trans2));
  help = GetString(foreign);
  XtOverrideTranslations(foreign,
			 XtParseTranslationTable(trans1));
  return(help);
}

/**********************************************************************
 *        some more complex X11 functions using popups etc.           *
 **********************************************************************/
/************************************************************
 * DisplayMessage : inactivate all buttons, open window and *
 *                  display 'message' in it. Pressing 'ok'  *
 *                  button will destroy window              *
 ************************************************************/
/**************************************************
 *        'ok' button callback function           *
 **************************************************/
/* ARGSUSED */
void Destroy(Widget w, XtPointer x, XtPointer y) {
  
  /*
   * set buttons sensitive
   */
  
  SetAllSensitive((Boolean) True);
  
  /*
   * destroy popup shell
   */
  
  XtDestroyWidget(XtParent(XtParent(w)));
}

/**************************************************
 *          begin of DisplayMessage()             *
 **************************************************/
void DisplayMessage(char *message) {
  Widget psh, form, mess, ok;
  psh = ComputePsh();
  
  /*
   * this is the message formWidget
   */
  
  form = XtVaCreateWidget("form", formWidgetClass, psh, NULL);
  
  /*
   * this is the message text labelWidget
   */
  
  mess = XtVaCreateManagedWidget(message, labelWidgetClass, form, NULL);
  XtVaSetValues(mess, XtNborderWidth, 0, NULL);
  
  /*
   * this is the ok command widget
   */
  
  ok = XtVaCreateManagedWidget("ok", commandWidgetClass, form, NULL);
  XtAddCallback(ok, XtNcallback, Destroy, NULL);
  XtVaSetValues(ok, XtNfromVert, mess, NULL);
  XtOverrideTranslations(ok, XtParseTranslationTable(trans5));
  
  /*
   * now lets popup the shell
   */
  
  XtManageChild(form);
  XtPopup(psh, XtGrabNonexclusive);
  XtSetKeyboardFocus(psh, ok);
  XtSetKeyboardFocus(top, ok);
}

/************************************************************
 * Quit : Called by menu 'File->Quit', exits X-Windows      *
 ************************************************************/
void Quit(void) {
  delete_all();
  free(head.filename);
  free(head.filename_bak);
  XtDestroyApplicationContext(app_con);
  exit(0);
}
